/******************************************************************************/
/* Program:								      */
/* 		MOXA UPort 2000 Device Utility				      */
/*									      */
/* History:								      */
/* 2008/03/21								      */
/*									      */
/* Author:								      */
/* Eric Lo, eric.lo@moxa.com						      */
/*									      */
/* Discription:								      */
/* The utility is to set or get the MOXA UPort 2000 configurations	      */
/*									      */
/******************************************************************************/

#include        <stdlib.h>
#include	<stdio.h>
#include        <string.h>
#include        <fcntl.h>
#include	<getopt.h>

#define MX_SET_INTERFACE	(404+21)
#define MX_GET_INTERFACE	(404+22)
#define MX_LOCATE_DEVICE	(404+23)
#define MX_GET_PID		(404+24)

#define MX_DEVICE_OFF		0x00
#define MX_DEVICE_ON		0x01
#define MX_RS232		1
#define MX_RS422		2
#define MX_RS485_2W		4
#define MX_RS485_4W		8
#define MX_UPORT_2450_PID	0x2450
#define MX_ON_TIME		100000
#define MX_OFF_TIME		400000
#define MX_UPORT_2450_ON_TIME	50000
#define MX_UPORT_2450_OFF_TIME	400000

#define OPT_OPEN		1
#define OPT_HELP		2
#define OPT_SET_INTERFACE	4
#define OPT_GET_INTERFACE	8
#define OPT_LOCATE_DEVICE	16

int mx_open_device(char *dev);
int mx_set_interface(int fd, char *dev, char *parm);
void mx_get_interface(int fd, char *dev);
void mx_locate_device(int fd);
void mx_show_help();

int	main(int argc,char *argv[])
{
	int fd, c, opt=0;
	char *dev, *parm;
	extern char *optarg;
	extern int optind;

	while((c = getopt(argc, argv, "glhs:")) != -1){
		switch(c){
			case 's':
				opt |= (OPT_OPEN | OPT_SET_INTERFACE);
				parm = optarg;
				break;
			case 'g':
				opt |= (OPT_OPEN | OPT_GET_INTERFACE);
				break;
			case 'l':
				opt |= (OPT_OPEN | OPT_LOCATE_DEVICE);
				break;
			case 'h':
			case '?':
				opt |= OPT_HELP;
				break;
		}
	}
	
	if(!opt)
		opt |= OPT_HELP;

	dev = argv[optind];

	if(opt & OPT_OPEN){
		if((fd = mx_open_device(dev)) < 0)
			exit(0);
	}

	if(opt & OPT_SET_INTERFACE)
		mx_set_interface(fd, dev, parm);
	else if(opt & OPT_GET_INTERFACE)
		mx_get_interface(fd,dev);
	else if(opt & OPT_LOCATE_DEVICE)
		mx_locate_device(fd);
	else if(opt & OPT_HELP)
		mx_show_help();

	close(fd);

	return 0;
}


int mx_open_device(char *dev)
{
	int fd;

	printf("mxustty: Open device %s ",dev);
	fd = open(dev,O_RDWR);

	if (fd < 0) {
		printf("error (%d)!\n",fd);
		return -1;
	}else
		printf("ok.\n");
	
	return fd;
}


int mx_set_interface(int fd, char *dev, char *parm)
{
	int mode = 0,ret;

	if(strncmp(parm,"RS232",5)==0)
		mode = MX_RS232;
	else if(strncmp(parm,"RS422",5)==0)
		mode = MX_RS422;
	else if(strncmp(parm,"RS4852W",8)==0)
		mode = MX_RS485_2W;
	else if(strncmp(parm,"RS4854W",8)==0)
		mode = MX_RS485_4W;

	if((ret=ioctl(fd,MX_SET_INTERFACE,mode))<0){
		printf("mxustty: Invalid operation of MOXA UPort 2000 series device.\n");
		return -5;
	}

	printf("mxustty: Set interface of %s ok.\n",dev);

	return 0;
}


void mx_get_interface(int fd, char *dev)
{
	int mode=0;

	ioctl(fd,MX_GET_INTERFACE,&mode);

	switch(mode){
		case MX_RS232:
			printf("mxustty: %s is set to RS-232 mode.\n",dev);
			break;
		case MX_RS422:
			printf("mxustty: %s is set to RS-422 mode.\n",dev);
			break;
		case MX_RS485_2W:
			printf("mxustty: %s is set to RS-485 2W mode.\n",dev);
			break;
		case MX_RS485_4W:
			printf("mxustty: %s is set to RS-485 4W mode.\n",dev);
			break;
	}
}


void mx_locate_device(int fd)
{
	char cret;
	int pid,on_time,off_time;

	fcntl(0,F_SETFL,O_NONBLOCK);
	printf("mxustty: Locating device...\n");
	printf("mxustty: Press Enter to Stop the Buzzer or LED...\n");

	ioctl(fd,MX_GET_PID,&pid);

	on_time = MX_ON_TIME;
	off_time = MX_OFF_TIME;

	if(pid == MX_UPORT_2450_PID){
		on_time = MX_UPORT_2450_ON_TIME;
		off_time = MX_UPORT_2450_OFF_TIME;
	}

	do{
		ioctl(fd,MX_LOCATE_DEVICE,MX_DEVICE_OFF);
		usleep(on_time);
		ioctl(fd,MX_LOCATE_DEVICE,MX_DEVICE_ON);
		usleep(off_time);

		cret = getchar();
		if(cret==10)
			break;

	}while(1);
}

void mx_show_help()
{
	printf("Usage: mxustty <operation> device\n");
	printf("\n");	
	printf("device: The UPort 2000 series device node.\n");
	printf("\n");	
	printf("operation	-h	  Help\n");
	printf("		-l	  Locate the device\n");
//	printf("		-g	  Get interface type\n"); //For further usage, Eric Lo, 2008/03/06
//	printf("		-s intf   Set interface type with options below\n"); //For further usage, Eric Lo, 2008/03/06
	printf("\n");	
//	printf("intf 		RS232     RS-232 mode\n");
//	printf("		RS422     RS-422 mode\n");
//	printf("		RS4852W   RS-485 2 wire mode\n");
//	printf("		RS4854W	  RS-485 4 wire mode\n");
}
